#include "ModelLoader.h"
#include <assimp/Importer.hpp>
#include <assimp/postprocess.h>
#include <assimp/scene.h>



void ProcessVertices(aiMesh* mesh, std::vector<YoggieEngine::Vertex>& out_vertices) {
	for (unsigned int i = 0; i < mesh->mNumVertices; i++) {
		YoggieEngine::Vertex v{};

		glm::vec3 vector;
		vector.x = mesh->mVertices[i].x;
		vector.y = mesh->mVertices[i].y;
		vector.z = mesh->mVertices[i].z;

		v.vertices = vector;

		if (mesh->mTextureCoords[0]) {
			glm::vec2 texCoord;

			texCoord.x = mesh->mTextureCoords[0][i].x;
			texCoord.y = mesh->mTextureCoords[0][i].y;

			v.uv = texCoord;

		}

		out_vertices.push_back(v);
	}
}


void ProcessIndices(aiMesh* mesh, std::vector<unsigned int>& out_indices) {
	for (unsigned int i = 0; i < mesh->mNumFaces; i++) {
		aiFace face = mesh->mFaces[i];
		if (face.mNumIndices < 3)
			continue;
		for (unsigned int j = 0; j < face.mNumIndices; j++) {
			out_indices.push_back(face.mIndices[j]);
		}
	}
}



YoggieEngine::Mesh processMesh(aiMesh* mesh, const aiScene* scene) {
	std::vector<unsigned int> indices;
	std::vector<YoggieEngine::Vertex> vertices;

	ProcessVertices(mesh, vertices);
	ProcessIndices(mesh, indices);

	YoggieEngine::Mesh result;
	result.vertices = vertices;
	result.elements = indices;

	return result;

}

std::vector<YoggieEngine::Mesh> processNode(aiNode* node, const aiScene* scene) {

	std::vector<YoggieEngine::Mesh> meshes = std::vector<YoggieEngine::Mesh>();

	for (unsigned int i = 0; i < node->mNumMeshes; i++) {
		aiMesh* mesh = scene->mMeshes[node->mMeshes[i]];
		meshes.push_back(processMesh(mesh, scene));
	}



	for (unsigned int i = 0; i < node->mNumChildren; i++) {
		auto m2 = processNode(node->mChildren[i], scene);

		for (auto m : m2) {
			meshes.push_back(m);
		}
	}

	return meshes;


}

Asset ModelLoader::LoadAsset(std::filesystem::path& path) {

	Assimp::Importer importer;

	const aiScene* scene = importer.ReadFile(path.string(), aiProcess_Triangulate | aiProcess_FlipUVs);

	aiNode* currentNode = scene->mRootNode;

	spdlog::debug("Loading meshes!");

	auto meshes = processNode(currentNode, scene);

	return Asset("Mesh");



}