#include "Viewport.h"

Viewport::Viewport(YoggieEngine::Scene& scene ):
    EditorWindow("SceneView"),
    renderer(YoggieEngine::RendererConfig{ 1200, 700, glm::vec3(0), true }),
    cam(glm::vec3(14.0f, 1.0f, 0.0f), glm::vec3(0.0f, 0.0f, 0.0f), 90),
    scene(scene)
{
    renderer.SetMainCamera(cam);

}

void Viewport::Draw() {

    auto group = scene.getReg().view<YoggieEngine::TransformComponent, YoggieEngine::Render3DComponent>();
    group.each([&](auto enity, YoggieEngine::TransformComponent& t, YoggieEngine::Render3DComponent& renderComponent) {
        renderer.Submit(renderComponent, t);
        });

    cam.Update();
    renderer.SetMainCamera(cam);
    renderer.Render(scene);

    ImVec2 WinPos = ImGui::GetWindowPos();
    ImVec2 ContentRegionMin = ImGui::GetWindowContentRegionMin();
    ImVec2 ContentRegionMax = ImGui::GetWindowContentRegionMax();
    ImVec2 ScreenSpaceMin = { ContentRegionMin.x + WinPos.x, ContentRegionMin.y + WinPos.y };
    ImVec2 ScreenSpaceMax = { ContentRegionMax.x + WinPos.x,ContentRegionMax.y + WinPos.y };


    ImGui::Image(
        (void*)(intptr_t)renderer.getCurrentFrameBuffer().GetColourAttachment(),
        ImVec2{ (float)ImGui::GetWindowWidth(),(float)ImGui::GetWindowHeight() }
    );


    

    ImGuizmo::Enable(true);
    ImGuizmo::SetOrthographic(false);
    ImGuizmo::SetDrawlist();

    float windowWidth = (float)ImGui::GetWindowWidth();
    float windowHeight = (float)ImGui::GetWindowHeight();
    ImGuizmo::SetRect(ImGui::GetWindowPos().x, ImGui::GetWindowPos().y, windowWidth, windowHeight);

    const auto& ProjMatrix = camera.projection;
    glm::mat4& cameraView = glm::mat4(1.0f);//glm::inverse(glm::translate(glm::mat4(1.0f) , cam.Position) * glm::toMat4(glm::quat(cam.Rotation)) );

    ImGuizmo::DrawGrid(glm::value_ptr(cameraView), glm::value_ptr(ProjMatrix), glm::value_ptr(cameraDelta), 100.0f);
    ImGuizmo::ViewManipulate(glm::value_ptr(cameraView), 1, ImGui::GetWindowPos(), {90,90}, 0x22CCCCCC);


    if (selected == nullptr)
        return;

    if (selected->isValid()) {
      
        auto& tc = selected->GetComponent<YoggieEngine::TransformComponent>();
        glm::mat4 transform = tc.GetTransform();

        ImGuizmo::Manipulate(
                    glm::value_ptr(cameraView),
            glm::value_ptr(ProjMatrix),
            ImGuizmo::TRANSLATE, ImGuizmo::LOCAL,
            glm::value_ptr(transform), nullptr, nullptr);
    }

   
}