#include <YoggieEngine.h>
#include "Application.h"
#include <imgui.h>
#include <backends/imgui_impl_opengl3.h>
#include <backends/imgui_impl_glfw.h>
#include "../../libs/guizmo/ImGuizmo.h"


namespace YoggieEngine {

	Application::Application(const std::string& name)
		: m_AppName(name)
	{
		appWindow = CreateNativeWindow(1200, 700, m_AppName.c_str());

		// Initialize engine should possibly happen here
		EngineInstrumentation::PerfomanceSamplerInit();

		// Initilize ImGui for this application 
		IMGUI_CHECKVERSION();
		ImGui::CreateContext();
		ImGuiIO& io = ImGui::GetIO();
		io.ConfigFlags |= ImGuiConfigFlags_::ImGuiConfigFlags_ViewportsEnable;
		io.ConfigFlags |= ImGuiConfigFlags_::ImGuiConfigFlags_DockingEnable;
		io.Fonts->AddFontFromFileTTF("build/Debug/Fonts/Roboto-Regular.ttf", 18);

		ImGui::StyleColorsDark();

		ImGui_ImplGlfw_InitForOpenGL((GLFWwindow*)appWindow->GetHandle(), true);
		ImGui_ImplOpenGL3_Init("#version 450");

		ImGuizmo::SetImGuiContext(ImGui::GetCurrentContext());
		ImGuizmo::SetOrthographic(true);


		init_inputSystem(appWindow);
	}


	void Application::Run() {
		
	}


	void Application::PollEvents() {
		appWindow->Poll();
	}

	void Application::SwapBuffers()
	{
		appWindow->SwapBuffers();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	}

	void Application::GuiBegin() {
		ImGui_ImplGlfw_NewFrame();
		ImGui_ImplOpenGL3_NewFrame();

		ImGui::NewFrame();
		ImGuizmo::BeginFrame();
		ImGui::DockSpaceOverViewport(ImGui::GetMainViewport());
		
	
	}

	void Application::GuiEnd() {
	
		ImGui::EndFrame();


		ImGui::Render();
		ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());


		if (ImGui::GetIO().ConfigFlags & ImGuiConfigFlags_ViewportsEnable)
		{
			GLFWwindow* last_context = glfwGetCurrentContext();
			ImGui::UpdatePlatformWindows();
			ImGui::RenderPlatformWindowsDefault();
			glfwMakeContextCurrent(last_context);
		}
	}

	void Application::PushLayer(Layer* layer)
	{
		AppLayerstack.PushLayer(layer);
		layer->OnAttach();
	}

	Application::~Application() {

		ImGui_ImplOpenGL3_Shutdown();
		ImGui_ImplGlfw_Shutdown();
		ImGui::DestroyContext();

		delete appWindow;

	}
}
