#pragma once
#include "Entity.h"
#include <glm/gtx/quaternion.hpp>
namespace YoggieEngine {
	struct IdentifierComponent {
		std::string name;
	};


	struct TransformComponent {
		glm::vec3 Position = glm::vec3(0.0f);
		glm::vec3 Rotation = glm::vec3(0.0f);
		glm::vec3 Scale = glm::vec3(1.0f);
		
		glm::mat4 LocalTransform = glm::mat4(1.0f);

		glm::mat4 GetTransform() const {
			glm::mat4 rotation = glm::toMat4(glm::quat(Rotation));
			return glm::translate(glm::mat4(1.0f), Position) * rotation * glm::scale(glm::mat4(1.0f), Scale);
		}


	};

	struct RelationComponent {
		Entity Parent;

		RelationComponent(Entity& entity) 
		{
			Parent = entity;
		}

	};


	struct LightComponent {
		glm::vec3 Color = glm::vec3(1.0f, .5f, .5f);
	};


	struct CameraComponent {
		glm::vec3 Position = glm::vec3(0);
		glm::vec3 Rotation = glm::vec3(0);
		float Zoom ;
	};


	struct ScriptComponent {
		std::string file; // TODO : replace with proper properties
	};


	struct RigidBody {
		float Mass;
		// Physics Material
	};

	struct BoxCollider {
		glm::vec3 extents;
	};

	struct PlaneCollider {

	};


	struct Render3DComponent {
		unsigned int VAO = 0;
		unsigned int IBO = 0;
		Mesh mesh;
		// TODO: becomes a material 
		glm::vec3 color;
		bool isStatic;

		Render3DComponent()
			: color(glm::vec3(1.0f, 0.0f, 0.0f)) , isStatic(true)
		{ 
		}
	};

	struct Render2DComponent {
		glm::vec3 Colour;
		
	};
};